@@ -58,8 +58,8 @@ static const struct pattern_flags
 Flags[] =
 {
   { 'A', MUTT_ALL,			0,		NULL },
-  { 'b', MUTT_BODY,		MUTT_FULL_MSG,	eat_regexp },
-  { 'B', MUTT_WHOLE_MSG,		MUTT_FULL_MSG,	eat_regexp },
+  { 'b', MUTT_BODY,  MUTT_FULL_MSG|MUTT_SEND_MODE_SEARCH, eat_regexp },
+  { 'B', MUTT_WHOLE_MSG,  MUTT_FULL_MSG|MUTT_SEND_MODE_SEARCH, eat_regexp },
   { 'c', MUTT_CC,			0,		eat_regexp },
   { 'C', MUTT_RECIPIENT,		0,		eat_regexp },
   { 'd', MUTT_DATE,		0,		eat_date },
@@ -70,7 +70,7 @@ Flags[] =
   { 'F', MUTT_FLAG,		0,		NULL },
   { 'g', MUTT_CRYPT_SIGN,		0,		NULL },
   { 'G', MUTT_CRYPT_ENCRYPT,	0,		NULL },
-  { 'h', MUTT_HEADER,		MUTT_FULL_MSG,	eat_regexp },
+  { 'h', MUTT_HEADER,  MUTT_FULL_MSG|MUTT_SEND_MODE_SEARCH, eat_regexp },
   { 'H', MUTT_HORMEL,		0,		eat_regexp },
   { 'i', MUTT_ID,			0,		eat_regexp },
   { 'k', MUTT_PGP_KEY,		0,		NULL },
@@ -257,6 +257,74 @@ cleanup:
   return match;
 }
 
+static int msg_search_sendmode (HEADER *h, pattern_t *pat)
+{
+  BUFFER *tempfile = NULL;
+  int match = 0;
+  char *buf = NULL;
+  size_t blen = 0;
+  FILE *fp = NULL;
+
+  if (pat->op == MUTT_HEADER || pat->op == MUTT_WHOLE_MSG)
+  {
+    tempfile = mutt_buffer_pool_get ();
+    mutt_buffer_mktemp (tempfile);
+    if ((fp = safe_fopen (mutt_b2s (tempfile), "w+")) == NULL)
+    {
+      mutt_perror (mutt_b2s (tempfile));
+      mutt_buffer_pool_release (&tempfile);
+      return 0;
+    }
+
+    mutt_write_rfc822_header (fp, h->env, h->content,
+                              MUTT_WRITE_HEADER_POSTPONE,
+                              0, 0);
+    fflush (fp);
+    fseek (fp, 0, 0);
+
+    while ((buf = mutt_read_line (buf, &blen, fp, NULL, 0)) != NULL)
+    {
+      if (patmatch (pat, buf) == 0)
+      {
+        match = 1;
+        break;
+      }
+    }
+
+    FREE (&buf);
+    safe_fclose (&fp);
+    unlink (mutt_b2s (tempfile));
+    mutt_buffer_pool_release (&tempfile);
+
+    if (match)
+      return match;
+  }
+
+  if (pat->op == MUTT_BODY || pat->op == MUTT_WHOLE_MSG)
+  {
+
+    if ((fp = safe_fopen (h->content->filename, "r")) == NULL)
+    {
+      mutt_perror (h->content->filename);
+      return 0;
+    }
+
+    while ((buf = mutt_read_line (buf, &blen, fp, NULL, 0)) != NULL)
+    {
+      if (patmatch (pat, buf) == 0)
+      {
+        match = 1;
+        break;
+      }
+    }
+
+    FREE (&buf);
+    safe_fclose (&fp);
+  }
+
+  return match;
+}
+
 static int eat_regexp (pattern_t *pat, int flags, BUFFER *s, BUFFER *err)
 {
   BUFFER buf;
@@ -399,6 +467,45 @@ static const char *getDate (const char *s, struct tm *t, BUFFER *err)
   char *p;
   time_t now = time (NULL);
   struct tm *tm = localtime (&now);
+  int iso8601=1;
+  int v=0;
+
+  for (v=0; v<8; v++)
+  {
+    if (s[v] && s[v] >= '0' && s[v] <= '9')
+    {
+      continue;
+    }
+    iso8601 = 0;
+    break;
+  }
+
+  if (iso8601)
+  {
+    int year;
+    int month;
+    int mday;
+    sscanf (s, "%4d%2d%2d", &year, &month, &mday);
+
+    t->tm_year = year;
+    if (t->tm_year > 1900)
+      t->tm_year -= 1900;
+    t->tm_mon = month - 1;
+    t->tm_mday = mday;
+
+    if (t->tm_mday < 1 || t->tm_mday > 31)
+    {
+      snprintf (err->data, err->dsize, _("Invalid day of month: %s"), s);
+      return NULL;
+    }
+    if (t->tm_mon < 0 || t->tm_mon > 11)
+    {
+      snprintf (err->data, err->dsize, _("Invalid month: %s"), s);
+      return NULL;
+    }
+
+    return (s+8);
+  }
 
   t->tm_mday = strtol (s, &p, 10);
   if (t->tm_mday < 1 || t->tm_mday > 31)
@@ -765,6 +872,41 @@ static const struct pattern_flags *lookup_tag (char tag)
   return NULL;
 }
 
+static const struct pattern_flags *lookup_op (int op)
+{
+  int i;
+
+  for (i = 0; Flags[i].tag; i++)
+    if (Flags[i].op == op)
+      return (&Flags[i]);
+  return NULL;
+}
+
+static void print_crypt_pattern_op_error (int op)
+{
+  const struct pattern_flags *entry;
+
+  entry = lookup_op (op);
+  if (entry)
+  {
+    /* L10N:
+       One of the crypt pattern operators: ~g, ~G, ~k, ~V
+       was invoked when Mutt was compiled without crypto support.
+       %c is the pattern character, i.e. "g".
+    */
+    mutt_error (_("Pattern operator '~%c' is disabled."), entry->tag);
+  }
+  else
+  {
+    /* L10N:
+       An unknown pattern operator was somehow invoked.  This
+       shouldn't be possible unless there is a bug.
+    */
+    mutt_error (_("error: unknown op %d (report this error)."), op);
+  }
+
+}
+
 static /* const */ char *find_matching_paren (/* const */ char *s)
 {
   int level = 1;
@@ -966,6 +1108,9 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	  mutt_pattern_free (&curlist);
 	  return NULL;
 	}
+        if (flags & MUTT_SEND_MODE_SEARCH)
+          tmp->sendmode = 1;
+
 	tmp->op = entry->op;
 
 	ps.dptr++; /* eat the operator and any optional whitespace */
@@ -1286,6 +1431,12 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
     case MUTT_BODY:
     case MUTT_HEADER:
     case MUTT_WHOLE_MSG:
+      if (pat->sendmode)
+      {
+        if (!h->content || !h->content->filename)
+          return 0;
+        return (pat->not ^ msg_search_sendmode (h, pat));
+      }
       /*
        * ctx can be NULL in certain cases, such as when replying to a message from the attachment menu and
        * the user has a reply-hook using "~h" (bug #2190).
@@ -1382,19 +1533,31 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
       return (pat->not ^ (h->collapsed && h->num_hidden > 1));
     case MUTT_CRYPT_SIGN:
       if (!WithCrypto)
-        break;
+      {
+        print_crypt_pattern_op_error (pat->op);
+        return 0;
+      }
       return (pat->not ^ ((h->security & SIGN) ? 1 : 0));
     case MUTT_CRYPT_VERIFIED:
       if (!WithCrypto)
-        break;
+      {
+        print_crypt_pattern_op_error (pat->op);
+        return 0;
+      }
       return (pat->not ^ ((h->security & GOODSIGN) ? 1 : 0));
     case MUTT_CRYPT_ENCRYPT:
       if (!WithCrypto)
-        break;
+      {
+        print_crypt_pattern_op_error (pat->op);
+        return 0;
+      }
       return (pat->not ^ ((h->security & ENCRYPT) ? 1 : 0));
     case MUTT_PGP_KEY:
       if (!(WithCrypto & APPLICATION_PGP))
-        break;
+      {
+        print_crypt_pattern_op_error (pat->op);
+        return 0;
+      }
       return (pat->not ^ ((h->security & PGPKEY) == PGPKEY));
     case MUTT_XLABEL:
       return (pat->not ^ (h->env->x_label && patmatch (pat, h->env->x_label) == 0));
@@ -1418,7 +1581,7 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
       return (pat->not ^ (h->thread && !h->thread->child));
   }
   mutt_error (_("error: unknown op %d (report this error)."), pat->op);
-  return (-1);
+  return (0);
 }
 
 static void quote_simple (BUFFER *tmp, const char *p)
